@@ -21,14 +21,13 @@ __author__ = 'Luigi Pirelli'
 __date__ = 'May 2015'
 __copyright__ = '(C) 2015, Luigi Pirelli'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
-
 import codecs
 import xml.sax.saxutils
 
-from osgeo import ogr
+import warnings
+with warnings.catch_warnings():
+    warnings.filterwarnings("ignore", category=DeprecationWarning)
+    from osgeo import ogr
 from qgis.core import (QgsProcessingFeedback,
                        QgsProcessingParameterMultipleLayers,
                        QgsProcessingParameterBoolean,
@@ -91,7 +90,7 @@ class Datasources2Vrt(QgisAlgorithm):
 
     def processAlgorithm(self, parameters, context, feedback):
         input_layers = self.parameterAsLayerList(parameters, self.INPUT, context)
-        unioned = self.parameterAsBool(parameters, self.UNIONED, context)
+        unioned = self.parameterAsBoolean(parameters, self.UNIONED, context)
         vrtPath = self.parameterAsOutputLayer(parameters, self.OUTPUT, context)
 
         vrtString = self.mergeDataSources2Vrt(input_layers,
