@@ -21,27 +21,18 @@ __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
-
 from pprint import pformat
 import time
 
-from qgis.PyQt.QtWidgets import QMessageBox
+from qgis.PyQt.QtWidgets import QPushButton, QDialogButtonBox
 from qgis.PyQt.QtCore import Qt, QCoreApplication
 
-from qgis.core import (QgsProcessingParameterDefinition,
-                       QgsProcessingParameterRasterDestination,
-                       QgsProcessingParameterVectorDestination,
-                       QgsProcessingParameterFeatureSink,
-                       QgsProcessingOutputLayerDefinition,
-                       QgsProcessingOutputHtml,
+from qgis.core import (QgsProcessingOutputHtml,
                        QgsProcessingOutputNumber,
                        QgsProcessingOutputString,
+                       QgsProcessingOutputBoolean,
                        QgsProject,
                        QgsProcessingMultiStepFeedback,
-                       Qgis,
                        QgsScopedProxyProgressTask)
 
 from qgis.gui import QgsProcessingAlgorithmDialogBase
@@ -70,6 +61,18 @@ class BatchAlgorithmDialog(QgsProcessingAlgorithmDialogBase):
         self.setMainWidget(BatchPanel(self, self.algorithm()))
         self.hideShortHelp()
 
+        self.btnRunSingle = QPushButton(QCoreApplication.translate('BatchAlgorithmDialog', "Run as Single Process…"))
+        self.btnRunSingle.clicked.connect(self.runAsSingle)
+        self.buttonBox().addButton(self.btnRunSingle, QDialogButtonBox.ResetRole) # reset role to ensure left alignment
+
+    def runAsSingle(self):
+        self.close()
+
+        from processing.gui.AlgorithmDialog import AlgorithmDialog
+        dlg = AlgorithmDialog(self.algorithm().create(), parent=iface.mainWindow())
+        dlg.show()
+        dlg.exec_()
+
     def runAlgorithm(self):
         alg_parameters = []
 
@@ -78,43 +81,8 @@ class BatchAlgorithmDialog(QgsProcessingAlgorithmDialogBase):
         load_layers = self.mainWidget().checkLoadLayersOnCompletion.isChecked()
         project = QgsProject.instance() if load_layers else None
 
-        for row in range(self.mainWidget().tblParameters.rowCount()):
-            col = 0
-            parameters = {}
-            for param in self.algorithm().parameterDefinitions():
-                if param.flags() & QgsProcessingParameterDefinition.FlagHidden or param.isDestination():
-                    continue
-                wrapper = self.mainWidget().wrappers[row][col]
-                parameters[param.name()] = wrapper.parameterValue()
-                if not param.checkValueIsAcceptable(wrapper.parameterValue()):
-                    self.messageBar().pushMessage("", self.tr('Wrong or missing parameter value: {0} (row {1})').format(
-                        param.description(), row + 1),
-                        level=Qgis.Warning, duration=5)
-                    return
-                col += 1
-            count_visible_outputs = 0
-            for out in self.algorithm().destinationParameterDefinitions():
-                if out.flags() & QgsProcessingParameterDefinition.FlagHidden:
-                    continue
-
-                count_visible_outputs += 1
-                widget = self.mainWidget().tblParameters.cellWidget(row, col)
-                text = widget.getValue()
-                if out.checkValueIsAcceptable(text):
-                    if isinstance(out, (QgsProcessingParameterRasterDestination,
-                                        QgsProcessingParameterVectorDestination,
-                                        QgsProcessingParameterFeatureSink)):
-                        # load rasters and sinks on completion
-                        parameters[out.name()] = QgsProcessingOutputLayerDefinition(text, project)
-                    else:
-                        parameters[out.name()] = text
-                    col += 1
-                else:
-                    self.messageBar().pushMessage("", self.tr('Wrong or missing output value: {0} (row {1})').format(
-                        out.description(), row + 1),
-                        level=Qgis.Warning, duration=5)
-                    return
-
+        for row in range(self.mainWidget().batchRowCount()):
+            parameters = self.mainWidget().parametersForRow(row, destinationProject=project, warnOnInvalid=True)
             alg_parameters.append(parameters)
 
         task = QgsScopedProxyProgressTask(self.tr('Batch Processing - {0}').format(self.algorithm().displayName()))
@@ -130,7 +98,7 @@ class BatchAlgorithmDialog(QgsProcessingAlgorithmDialogBase):
             try:
                 self.showLog()
                 self.repaint()
-            except:
+            except Exception:  # FIXME which one?
                 pass
 
             start_time = time.time()
@@ -169,7 +137,7 @@ class BatchAlgorithmDialog(QgsProcessingAlgorithmDialogBase):
                     feedback.pushInfo(self.tr('Results:'))
                     feedback.pushCommandInfo(pformat(results))
                     feedback.pushInfo('')
-                    algorithm_results.append(results)
+                    algorithm_results.append({'parameters': parameters, 'results': results})
                 else:
                     break
 
@@ -183,7 +151,7 @@ class BatchAlgorithmDialog(QgsProcessingAlgorithmDialogBase):
 
     def finish(self, algorithm_results):
         for count, results in enumerate(algorithm_results):
-            self.loadHTMLResults(results, count)
+            self.loadHTMLResults(results['results'], count)
 
         self.createSummaryTable(algorithm_results)
         self.mainWidget().setEnabled(True)
@@ -199,7 +167,7 @@ class BatchAlgorithmDialog(QgsProcessingAlgorithmDialogBase):
         createTable = False
 
         for out in self.algorithm().outputDefinitions():
-            if isinstance(out, (QgsProcessingOutputNumber, QgsProcessingOutputString)):
+            if isinstance(out, (QgsProcessingOutputNumber, QgsProcessingOutputString, QgsProcessingOutputBoolean)):
                 createTable = True
                 break
 
@@ -208,12 +176,24 @@ class BatchAlgorithmDialog(QgsProcessingAlgorithmDialogBase):
 
         outputFile = getTempFilename('html')
         with codecs.open(outputFile, 'w', encoding='utf-8') as f:
-            for res in algorithm_results:
-                f.write('<hr>\n')
+            for i, res in enumerate(algorithm_results):
+                results = res['results']
+                params = res['parameters']
+                if i > 0:
+                    f.write('<hr>\n')
+                f.write(self.tr('<h3>Parameters</h3>\n'))
+                f.write('<table>\n')
+                for param in self.algorithm().parameterDefinitions():
+                    if not param.isDestination():
+                        if param.name() in params:
+                            f.write('<tr><th>{}</th><td>{}</td></tr>\n'.format(param.description(), params[param.name()]))
+                f.write('</table>\n')
+                f.write(self.tr('<h3>Results</h3>\n'))
+                f.write('<table>\n')
                 for out in self.algorithm().outputDefinitions():
-                    if isinstance(out, (QgsProcessingOutputNumber, QgsProcessingOutputString)) and out.name() in res:
-                        f.write('<p>{}: {}</p>\n'.format(out.description(), res[out.name()]))
-            f.write('<hr>\n')
+                    if out.name() in results:
+                        f.write('<tr><th>{}</th><td>{}</td></tr>\n'.format(out.description(), results[out.name()]))
+                f.write('</table>\n')
 
         resultsList.addResult(icon=self.algorithm().icon(),
                               name='{} [summary]'.format(self.algorithm().name()), timestamp=time.localtime(),
