@@ -21,10 +21,6 @@ __author__ = 'Matthias Kuhn'
 __date__ = 'January 2016'
 __copyright__ = '(C) 2016, Matthias Kuhn'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = ':%H$'
-
 
 import qgis  # NOQA switch sip api
 
@@ -36,6 +32,7 @@ import shutil
 import glob
 import hashlib
 import tempfile
+import re
 
 from osgeo.gdalconst import GA_ReadOnly
 from numpy import nan_to_num
@@ -43,6 +40,7 @@ from copy import deepcopy
 
 from qgis.core import (QgsVectorLayer,
                        QgsRasterLayer,
+                       QgsCoordinateReferenceSystem,
                        QgsFeatureRequest,
                        QgsMapLayer,
                        QgsProject,
@@ -70,7 +68,7 @@ class AlgorithmsTest(object):
         This is the main test function. All others will be executed based on the definitions in testdata/algorithm_tests.yaml
         """
         with open(os.path.join(processingTestDataPath(), self.test_definition_file()), 'r') as stream:
-            algorithm_tests = yaml.load(stream)
+            algorithm_tests = yaml.load(stream, Loader=yaml.SafeLoader)
 
         if 'tests' in algorithm_tests and algorithm_tests['tests'] is not None:
             for idx, algtest in enumerate(algorithm_tests['tests']):
@@ -84,7 +82,22 @@ class AlgorithmsTest(object):
         :param defs: A python dict containing a test algorithm definition
         """
         self.vector_layer_params = {}
-        QgsProject.instance().removeAllMapLayers()
+        QgsProject.instance().clear()
+
+        if 'project' in defs:
+            full_project_path = os.path.join(processingTestDataPath(), defs['project'])
+            project_read_success = QgsProject.instance().read(full_project_path)
+            self.assertTrue(project_read_success, 'Failed to load project file: ' + defs['project'])
+
+        if 'project_crs' in defs:
+            QgsProject.instance().setCrs(QgsCoordinateReferenceSystem(defs['project_crs']))
+        else:
+            QgsProject.instance().setCrs(QgsCoordinateReferenceSystem())
+
+        if 'ellipsoid' in defs:
+            QgsProject.instance().setEllipsoid(defs['ellipsoid'])
+        else:
+            QgsProject.instance().setEllipsoid('')
 
         params = self.load_params(defs['params'])
 
@@ -96,7 +109,7 @@ class AlgorithmsTest(object):
             for param in zip(alg.parameterDefinitions(), params):
                 parameters[param[0].name()] = param[1]
         else:
-            for k, p in list(params.items()):
+            for k, p in params.items():
                 parameters[k] = p
 
         for r, p in list(defs['results'].items()):
@@ -108,6 +121,9 @@ class AlgorithmsTest(object):
             exec(('\n'.join(defs['expectedFailure'][:-1])), globals(), locals())
             expectFailure = eval(defs['expectedFailure'][-1])
 
+        if 'expectedException' in defs:
+            expectFailure = True
+
         # ignore user setting for invalid geometry handling
         context = QgsProcessingContext()
         context.setProject(QgsProject.instance())
@@ -143,7 +159,7 @@ class AlgorithmsTest(object):
         if isinstance(params, list):
             return [self.load_param(p) for p in params]
         elif isinstance(params, dict):
-            return {key: self.load_param(p, key) for key, p in list(params.items())}
+            return {key: self.load_param(p, key) for key, p in params.items()}
         else:
             return params
 
@@ -185,7 +201,8 @@ class AlgorithmsTest(object):
                 basename = os.path.basename(param['name'])
             else:
                 basename = os.path.basename(param['name'][0])
-            filepath = os.path.join(outdir, basename)
+
+            filepath = self.uri_path_join(outdir, basename)
             return filepath
         elif param['type'] == 'rasterhash':
             outdir = tempfile.mkdtemp()
@@ -196,18 +213,22 @@ class AlgorithmsTest(object):
                 basename = 'raster.tif'
             filepath = os.path.join(outdir, basename)
             return filepath
+        elif param['type'] == 'directory':
+            outdir = tempfile.mkdtemp()
+            return outdir
 
         raise KeyError("Unknown type '{}' specified for parameter".format(param['type']))
 
     def load_layers(self, id, param):
         layers = []
-        if param['type'] in ('vector', 'table') and isinstance(param['name'], str):
-            layers.append(self.load_layer(id, param))
-        elif param['type'] in ('vector', 'table'):
-            for n in param['name']:
-                layer_param = deepcopy(param)
-                layer_param['name'] = n
-                layers.append(self.load_layer(id, layer_param))
+        if param['type'] in ('vector', 'table'):
+            if isinstance(param['name'], str) or 'uri' in param:
+                layers.append(self.load_layer(id, param))
+            else:
+                for n in param['name']:
+                    layer_param = deepcopy(param)
+                    layer_param['name'] = n
+                    layers.append(self.load_layer(id, layer_param))
         else:
             layers.append(self.load_layer(id, param))
         return layers
@@ -216,6 +237,7 @@ class AlgorithmsTest(object):
         """
         Loads a layer which was specified as parameter.
         """
+
         filepath = self.filepath_from_param(param)
 
         if 'in_place' in param and param['in_place']:
@@ -254,13 +276,28 @@ class AlgorithmsTest(object):
         if 'location' in param and param['location'] == 'qgs':
             prefix = unitTestDataPath()
 
-        return os.path.join(prefix, param['name'])
+        if 'uri' in param:
+            path = param['uri']
+        else:
+            path = param['name']
+
+        return self.uri_path_join(prefix, path)
+
+    def uri_path_join(self, prefix, filepath):
+        if filepath.startswith('ogr:'):
+            if not prefix[-1] == os.path.sep:
+                prefix += os.path.sep
+            filepath = re.sub(r"dbname='", "dbname='{}".format(prefix), filepath)
+        else:
+            filepath = os.path.join(prefix, filepath)
+
+        return filepath
 
     def check_results(self, results, context, params, expected):
         """
         Checks if result produced by an algorithm matches with the expected specification.
         """
-        for id, expected_result in list(expected.items()):
+        for id, expected_result in expected.items():
             if expected_result['type'] in ('vector', 'table'):
                 if 'compare' in expected_result and not expected_result['compare']:
                     # skipping the comparison, so just make sure output is valid
@@ -317,6 +354,11 @@ class AlgorithmsTest(object):
                 result_filepath = results[id]
 
                 self.assertFilesEqual(expected_filepath, result_filepath)
+            elif 'directory' == expected_result['type']:
+                expected_dirpath = self.filepath_from_param(expected_result)
+                result_dirpath = results[id]
+
+                self.assertDirectoriesEqual(expected_dirpath, result_dirpath)
             elif 'regex' == expected_result['type']:
                 with open(results[id], 'r') as file:
                     data = file.read()
